from ljson.slapdash.mem import Table
from ljson.slapdash import SlapdashHeader

from .component import AbstractComponent

class KeyValueStore(AbstractComponent):
	component_type = "kvs"
	def __init__(self, bunker, name, password):
		AbstractComponent.__init__(self, bunker, name, password)

		file_, type_ = bunker._load_component(name, password)
		self._content = Table.from_file(file_)
		self._open = True

	@classmethod
	def new(cls, bunker, name, password):
		bunker._add_component(name, password, KeyValueStore.component_type)

		file_, type_ = bunker._load_component(name, password)
		table = Table(SlapdashHeader({}), [])
		table.save(file_)
		bunker._save_component(name, password, file_)

		return cls(bunker, name, password)

	def additem(self, key, value):
		if(not self._open):
			raise IOError("KeyValueStore is closed")
		self._content.additem({"key": key, "value": value})
	def getitem(self, key):
		if(not self._open):
			raise IOError("KeyValueStore is closed")
		return self._content[{"key": key}]["value"][0]

	def write_back(self):
		if(not self._open):
			raise IOError("KeyValueStore is closed")
		file_, type_ = self._bunker._load_component(self._name, self._password)
		self._content.save(file_)
		self._bunker._save_component(self._name, self._password, file_)

	@property
	def closed(self):
		return not self._open

	def close(self, notify_bunker=True):
		self.write_back()
		self._open = False
		self._name = ""
		self._password = b""
		del(self._content)
		if(notify_bunker):
			self._bunker.notify_component_is_closed(self)
	
