import tarfile
import sys
import io
import tempfile
import os

class BunkeredFile(io.RawIOBase):
	def __init__(self
			, file_
			, name
			, size=None
			, isvirtual=False
			, ismem=False
			, bunker=None):
		self._file = file_
		self.isvirtual = isvirtual
		self.ismem = ismem

		self.name = name
		self.size = size
		self.bunker = bunker



	# Those are just the methods inherited from RawIOBase.
	# We need them to operate like a proper file.
	def read(self, size=-1):
		return self._file.read(size)
	def readinto(self, b):
		return self._file.readinto(b)
	def readall(self):
		return self._file.readall()
	def write(self, b):
		return self._file.write(b)
	def close(self, writeback=True):
		if(writeback and self.bunker):
			self.bunker.writeback_file(self.name)
		return self._file.close()
	def fileno(self):
		return self._file.fileno()
	def flush(self):
		return self._file.flush()
	def isatty(self):
		return self._file.isatty()
	def readable(self):
		return self._file.readable()
	def readline(self, size=-1):
		return self._file.readline(size)
	def readlines(self, hint=-1):
		return self._file.readlines(hint)
	def seek(self, offset, whence=0):
		return self._file.seek(offset, whence)
	def seekable(self):
		return self._file.seekable()
	def tell(self):
		return self._file.tell()
	def truncate(self, size=None):
		return self._file.truncate(size)
	def writable(self):
		return self._file.writable()
	def writelines(self, lines):
		return self._file.writelines(lines)
	def __del__(self):
		del(self._file)
	@property
	def closed(self):
		return self._file.closed


	# Classmethods to construct new BunkeredFiles.
	@classmethod
	def from_file(cls, file_, name):
		"""
		Construct a new BunkeredFile from an existing file on disk.
		This is used to bunker files and directories from the disk.

		DO NOT (!) use this to construct BunkeredFiles from BytesIO.
		"""
		size = None
		if(file_.seekable()):
			file_.seek(0, 2)
			size = file_.tell()
			file_.seek(0, 0)
		return cls(file_, name, size=size, isvirtual=False, ismem=False)

	@classmethod
	def empty(cls, name
			, length_hint=None
			, max_in_memory_bytes=2**20
			, mktempfile=tempfile.TemporaryFile):
		"""
		Construct a new BunkeredFile that uses BytesIO in the background.

		This is used either when loading data from a remote ressource or for 
		databases.
		"""
		if(length_hint is not None
			and length_hint > max_in_memory_bytes):
			file_ = mktempfile()
			ismem = False
		else:
			file_ = io.BytesIO()
			ismem = True
		return cls(file_, name, size=None, isvirtual=True, ismem=ismem)

	@classmethod
	def from_BytesIO(cls, bytes_, name):
		size = None
		if(bytes_.seekable()):
			bytes_.seek(0, 2)
			size = bytes_.tell()
			bytes_.seek(0, 0)
		return cls(bytes_, name, size=size, isvirtual=True, ismem=True)

	@classmethod
	def from_tar(cls
			, tarfile
			, tarinfo
			, rewriteable_tar_file=None
			, max_in_memory_bytes=2**20
			, mktempfile=tempfile.TemporaryFile):
		"""
		Load the file specified by ``tarinfo`` from ``tarfile``. If the size of the file
		is smaller than ``max_in_memory_bytes`` it will be loaded into memory which increases speed 
		security because an attacker cannot find the file on-disk.

		If the size is greated than ``max_in_memory_bytes`` it will be loaded into a file created by
		``mstemp``. In order to increase security this function should create a file that is hard to find.
		Having the file on the disk is a security vulnerability, because while the file is open it can be found.

		Also this method is vulnerable to malformed tar files.
		See `the docs of tarfile <https://docs.python.org/3.5/library/tarfile.html#tarfile.TarFile.extractall>`_.
		"""
		if(tarinfo.size > max_in_memory_bytes):
			file_ = mktempfile()
			ismem = False
		else:
			file_ = io.BytesIO()
			ismem = True

		with tarfile.extractfile(tarinfo) as fin:
			file_.write(fin.read())
		file_.seek(0, 0)
		return cls(file_, tarinfo.name, size=tarinfo.size, ismem=ismem, isvirtual=True, bunker=rewriteable_tar_file)

	def __len__(self):
		if(not self._file.seekable()):
			return 0
		rewind = self._file.tell()
		self._file.seek(0, 2)
		length = self._file.tell()
		self._file.seek(rewind, 0)
		return length

	def rewind(self):
		self._file.seek(0, 0)
		
	def __bool__(self):
		return True
